# -*- coding: utf-8 -*-

from ZODB.blob import Blob
from ZODB.interfaces import BlobError
from persistent import Persistent

from zope.interface import implements
from zope.location.interfaces import ILocation
from zope.contenttype import guess_content_type
from zope.component import queryMultiAdapter, getUtility
from zope.schema.fieldproperty import FieldProperty

from dolmen.file import clean_filename
from dolmen.blob import IBlobFile, IFileStorage


class StorageError(Exception):
    """An error raised if the storage failed.
    """


class BlobValue(object):
    """A BlobValue is using a ZODB Blob to store data. It handles both
    the zope.app.file and zope.file features. It can be used as a blob
    attribute, for more complex object. It can also be used as a mixin
    with a Persistent class.
    """
    implements(IBlobFile)
    
    filename = FieldProperty(IBlobFile['filename'])
    mimeType = FieldProperty(IBlobFile['mimeType'])
    parameters = FieldProperty(IBlobFile['parameters'])
    
    def __init__(self, data='', contentType='',
                 filename=None, parameters=None):

        if filename:
            filename = clean_filename(filename)
            self.filename = filename

        if not contentType and filename:
            self.mimeType, enc = guess_content_type(name=filename)
        elif not contentType:
            self.mimeType = "application/octet-stream"
        else:
            self.mimeType = contentType

        if parameters is None:
            parameters = {}
        else:
            parameters = dict(parameters)
        self.parameters = parameters

        self._blob = Blob()
        self.data = data

    @property
    def contentType(self):
        return self.mimeType

    def open(self, mode="r"):
        return self._blob.open(mode)

    def openDetached(self):
        return file(self._blob.committed(), 'rb')

    def __len__(self):
        if self._blob == "":
            return 0
        reader = self._blob.open()
        reader.seek(0,2)
        size = reader.tell()
        reader.close()
        return size

    getSize = __len__
    
    @property
    def size(self):
        return int(self.getSize())

    @apply
    def data():
        """The blob property using a IFileStorage adapter
        to write down the value.
        """
        def get(self):
            blob = self._blob.open('r')
            data = blob.read()
            blob.close()
            return data
        
        def set(self, value):
            stored = queryMultiAdapter((self._blob, value), IFileStorage)
            if stored is not True:
                raise StorageError(
                    "An error occured during the blob storage. Check the "
                    "value type (%r). This value should implement IFile, "
                    "IString or IUnicode (see `dolmen.builtins`)."
                    % value.__class__)

        return property(get, set)

    @property
    def physical_path(self):
        try:
            filename = self._blob.committed()
        except BlobError:
            # We retry, the data has now been commited
            # if possible by the ZODB blob.
            try:
                filename = self._blob.committed()
            except BlobError:
                # The retry failed, we return None.
                return None
        return filename


class BlobFile(Persistent, BlobValue):
    """A INameFile component using a ZODB Blob to store the data.
    """
    implements(ILocation)
    
    __name__ = None
    __parent__ = None



# -*- coding: utf-8 -*-

import grokcore.view as grok
from dolmen.blob import IBlobFile
from zope.file.download import DownloadResult, getHeaders


class FilePublisher(grok.View):
    grok.name('file_publish')
    grok.context(IBlobFile)

    def update(self):
        for k, v in getHeaders(self.context,
                               downloadName=self.context.filename,
                               contentDisposition="attachment"):
            self.request.response.setHeader(k, v)
 
    def render(self):
        return DownloadResult(self.context)



# -*- coding: utf-8 -*-

from dolmen import builtins
from dolmen.file import INamedFile
from zope.file.interfaces import IFile
from zope.publisher.browser import FileUpload
from zope.interface import Interface, Attribute, classImplements


class IBlobFile(INamedFile, IFile):
    """A marker interface for file using ZODB blobs.
    """
    physical_path = Attribute("Physical path of the stored file.")
    

class IFileStorage(Interface):
    """A component dedicated to store items.
    """
    def __call__(file, data):
        """Persists the data in the file element.
        """


class IFileUpload(builtins.IFile):
    """Defines an Upload object created by zope.publisher
    for data elements in forms.
    """
classImplements(FileUpload, IFileUpload)



# -*- coding: utf-8 -*-

import shutil
import grokcore.component as grok

from ZODB.interfaces import IBlob
from zope.interface import Interface
from dolmen.blob import IFileStorage, IBlobFile
from dolmen.builtins import interfaces as base

CHUNK = 1 << 12


@grok.implementer(IFileStorage)
@grok.adapter(IBlob, base.IString)
def string_blob(blob, data):
    fp = blob.open('w')
    fp.write(data)
    fp.close()
    return True


@grok.implementer(IFileStorage)
@grok.adapter(IBlob, base.IUnicode)
def unicode_blob(blob, data):
    data = data.encode('UTF-8')
    string_blob(blob, data)
    return True


@grok.implementer(IFileStorage)
@grok.adapter(IBlob, IBlobFile)
def blob_to_blob(target, source):
    fsrc = source.open('r')
    fdst = target.open('w')
    shutil.copyfileobj(fsrc, fdst)
    fdst.close()
    fsrc.close()
    return True


@grok.implementer(IFileStorage)
@grok.adapter(IBlob, Interface)
def filetype_blob(blob, data):
    try:
        data.seek(0)
        fp = blob.open('w')
        block = data.read(CHUNK)
        while block:
            fp.write(block)
            block = data.read(CHUNK)
        fp.close()
    except AttributeError:
        return False
    return True



from setuptools import setup, find_packages
from os.path import join

name = 'dolmen.blob'
version = '0.3'
readme = open(join('src', 'dolmen', 'blob', "README.txt")).read()
history = open(join('docs', 'HISTORY.txt')).read()

install_requires = [
    'setuptools',
    'ZODB3>=3.9.0',
    'zope.file',
    'zope.mimetype',
    'dolmen.file>=0.3.2',
    'dolmen.builtins>=0.2',
    'grokcore.view',
    'grokcore.component',
    ]

tests_require = install_requires + [
    'zope.size',
    'zope.testing',
    'zope.filerepresentation',
    'zope.app.testing',
    'zope.app.zcmlfiles',
    ]

setup(name = name,
      version = version,
      description = 'Dolmen zodb blob handlers',
      long_description = readme + '\n\n' + history,
      keywords = 'Grok Zope3 CMS Dolmen',
      author = 'Souheil Chelfouh',
      author_email = 'trollfot@gmail.com',
      url = '',
      license = 'GPL',
      packages=find_packages('src', exclude=['ez_setup']),
      package_dir={'': 'src'},
      namespace_packages = ['dolmen'],
      include_package_data = True,
      platforms = 'Any',
      zip_safe = False,
      tests_require = tests_require,
      install_requires = install_requires,
      extras_require = {'test': tests_require},
      test_suite="dolmen.blob",
      classifiers = [
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Framework :: Zope3',
        'Intended Audience :: Other Audience',
        'License :: OSI Approved :: GNU General Public License (GPL)',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
      ],
)
